﻿module DocumentationMenu {

    interface NodeData {
        name: string;
        description: string;
        objectId: string;
        nodes: NodeData[];
    }
    interface Tree {
        nodes: NodeData[];
    }
    interface Input {
        tree: Tree;
    }

    class NodeModel {

        isExpanded = ko.observable(false);
        description = ko.observable('');
        name = ko.observable('');
        nodes = ko.observable<NodeModel[]>();
        nodeCssClass: KnockoutComputed<object>;

        constructor(data: NodeData) {
            if (data == null)
                throw "null data";

            this.description(data.description);
            this.name(data.name);
            this.nodeCssClass = ko.pureComputed(
                () => {
                    return {
                        'tree-icon': true,
                        'minus-sign': this.isExpanded() && this.nodes().length > 0,
                        'plus-sign': !this.isExpanded() && this.nodes().length > 0,
                        //'square': this.nodes().length === 0,
                        'no-icon': this.nodes().length === 0
                    };
                }
            );

            this.nodes(data.nodes.makeIterable().select(n => new NodeModel(n)).toArray());
        }

        toggleVisibility() : void {
            this.isExpanded(!this.isExpanded());
        };
    }

    class TreeModel {
        nodes = ko.observable<NodeModel[]>();
        constructor(tree: Tree) {
            this.nodes(tree.nodes.makeIterable().select(n => new NodeModel(n)).toArray());
        }
    }

    class PageModel {

        tree = ko.observable<TreeModel>();

        constructor(input: Input) {
            this.tree(new TreeModel(input.tree));
        }
    }

    export function Init(input: Input) {
        var model = new PageModel(input);
        
        ko.applyBindings(model);
    }
}

var rubbish = {
    tree: {
        nodes: [
            {
                name: 'HDARS 1',
                description: 'hello dears',
                objectId: '',
                nodes: [
                    {
                        name: 'Child 1',
                        description: 'test description!',
                        objectId: '',
                        nodes: [
                        ]
                    },
                    {
                        name: 'Child 2',
                        description: 'test description! asd',
                        objectId: '',
                        nodes: [
                            {
                                name: 'Child Child 1',
                                description: 'test description test description ',
                                objectId: '',
                                nodes: [
                                ]
                            }
                        ]
                    }
                ]
            },
            {
                name: 'Hdars 2',
                description: 'for sirs, description of long for still rubbish',
                objectId: '',
                nodes: [
                ]
            },
            {
                name: 'Hdars 3',
                description: '',
                objectId: '',
                nodes: [
                ]
            }
        ]
    }
};

$(() => DocumentationMenu.Init(rubbish));